import sys
from argparse import ArgumentParser, Namespace

from . import normalize


def setup_argument_subparser(parser: ArgumentParser) -> None:
    subsubparsers = parser.add_subparsers(dest="subsubcommand")
    subsubparsers.add_parser("normalize")


def main(args: Namespace) -> None:
    if args.subsubcommand == "normalize":
        sys.stdout.write(normalize(sys.stdin.read()))
    else:
        args.print_help()



import math
import re
import sys
import unicodedata
from typing import Optional, Union

import regex

from .exceptions import UnknownEncodingError

if sys.version_info < (3, 9):
    from typing import Pattern

    PatternT = Union[Pattern[str], regex.regex.Pattern]
else:
    PatternT = Union[re.Pattern[str], regex.regex.Pattern[str]]

codec_list = [
    "utf_8",
    "utf_8_sig",
    "utf_7",
    "utf_16",
    "utf_16_be",
    "utf_32",
    "utf_32_be",
    "euc_jp",
    "gbk",
    "iso2022_jp",
]


def guess_encoding(x: bytes, hint: Optional[Union[str, PatternT]] = None) -> str:
    if hint is None:
        return guess_encoding(
            x,
            regex.compile(
                r'[\p{Script=Han}\u3041-\u309F\u30A1-\u30FF\uFF01-\uFF0F\uFF1A-\uFF20'
                r'\uFF3B-\uFF40\uFF5B-\uFF65\u3000-\u303F]+'
            ),
        )
    if isinstance(hint, str):
        return guess_encoding(x, re.compile(hint))
    for c in codec_list:
        try:
            buf = x.decode(c)
            if hint.match(buf) is not None:
                return c
        except ValueError:
            ...
    raise UnknownEncodingError()


def decode(x: bytes) -> str:
    return x.decode(guess_encoding(x))


numeric_string_regex = re.compile(
    r"^(?P<minus>-)?"
    + "".join(
        (
            f"(?:(?P<{dig}>([0-9.]*千)?([0-9.]*百)?([0-9.]*十)?([0-9.]*)?)?{dig})?"
            for dig in [
                "無量大数",
                "不可思議",
                "那由他",
                "阿僧祇",
                "恒河沙",
                "極",
                "載",
                "正",
                "澗",
                "溝",
                "穣",
                "𥝱",
                "垓",
                "京",
                "兆",
                "億",
                "万",
            ]
        )
    )
    + r"(?P<一>([0-9.]*千)?([0-9.]*百)?([0-9.]*十)?([0-9.]*)?)?$",
)

four_digits_string_regex = re.compile(r"^((?P<千>[0-9.]*)千)?((?P<百>[0-9.]*)百)?((?P<十>[0-9.]*)十)?((?P<一>[0-9.]*))?$")
trans_map = str.maketrans(
    {
        "１": "1",
        "２": "2",
        "３": "3",
        "４": "4",
        "５": "5",
        "６": "6",
        "７": "7",
        "８": "8",
        "９": "9",
        "０": "0",
        "一": "1",
        "二": "2",
        "三": "3",
        "四": "4",
        "五": "5",
        "六": "6",
        "七": "7",
        "八": "8",
        "九": "9",
        "〇": "0",
        "壱": "1",
        "弐": "2",
        "参": "3",
        "零": "0",
    }
)


def to_numeric(x: str) -> Union[float, int]:
    if not isinstance(x, str):
        raise TypeError(f"to_numeric() argument must be a string, not '{type(x)}'")
    if len(x) == 0:
        raise ValueError(f"invalid literal: {x}")
    x_ = x.translate(trans_map).replace(",", "")
    m = numeric_string_regex.match(x_)
    if m is None or len(m[0]) == 0:
        raise ValueError(f"invalid literal: {x}")
    buf = 0
    valid = False
    for i, k in enumerate(
        [
            "一",
            "万",
            "億",
            "兆",
            "京",
            "垓",
            "𥝱",
            "穣",
            "溝",
            "澗",
            "正",
            "載",
            "極",
            "恒河沙",
            "阿僧祇",
            "那由他",
            "不可思議",
            "無量大数",
        ]
    ):
        digits = m[k]
        if digits is None:
            continue
        if len(digits) > 0:
            buf += _parse_sen_digits(digits) * (10000**i)
            valid = True
    if not valid:
        raise ValueError(f"invalid literal: {x}")
    return (-1 if m["minus"] is not None else 1) * buf


def _parse_sen_digits(x: str) -> Union[float, int]:
    m = four_digits_string_regex.match(x)
    if m is not None:
        s = (
            (0 if m["千"] is None else (1 if len(m["千"]) == 0 else float(m["千"]))) * 1000
            + (0 if m["百"] is None else (1 if len(m["百"]) == 0 else float(m["百"]))) * 100
            + (0 if m["十"] is None else (1 if len(m["十"]) == 0 else float(m["十"]))) * 10
            + (0 if m["一"] is None or len(m["一"]) == 0 else float(m["一"]))
        )
        if s == math.floor(s):
            return int(s)
        return s
    return 0


normalize_trans_map = str.maketrans(
    {
        "˗": "-",
        "֊": "-",
        "‐": "-",
        "‑": "-",
        "‒": "-",
        "–": "-",
        "⁃": "-",
        "⁻": "-",
        "₋": "-",
        "−": "-",
        "　": " ",
        "\u200b": " ",
        "\ufeff": " ",
        "\t": " ",
        "“": "\"",
        "”": "\"",
        "‘": "'",
        "’": "'",
    }
)

parentheses_left = re.compile(r"([^\s(])\(")
parentheses_right = re.compile(r"\)([^\s)])")


def normalize(x: str, newline_to_space: bool = False, remove_multiple_spaces: bool = False) -> str:
    normalized_string = parentheses_right.sub(
        r") \1", parentheses_left.sub(r"\1 (", unicodedata.normalize("NFKC", x).translate(normalize_trans_map))
    )
    if newline_to_space:
        normalized_string = re.sub(r"[\n\r]", " ", normalized_string)
    if remove_multiple_spaces:
        return re.sub(r" +", " ", normalized_string)
    return normalized_string



from jittok import Callable as jCallable


class ConcreteCallable(jCallable[int, str]):
    def __call__(self, x: int) -> str:
        return str(x)


def test_jittok_callable_is_callable() -> None:
    sut = ConcreteCallable()
    assert callable(sut)



from unittest.mock import call

from pytest_mock import MockerFixture

from jittok import cli


def test_setup_argument_parser(mocker: MockerFixture) -> None:
    jptext_cli = mocker.patch("jittok.cli.jptext_cli")
    ArgumentParser = mocker.patch("jittok.cli.ArgumentParser")
    parser = ArgumentParser.return_value
    subparsers = parser.add_subparsers.return_value
    actual = cli.setup_argument_parser()
    jptext_cli.setup_argument_subparser.assert_called_once_with(subparsers.add_parser.return_value)
    parser.add_subparsers.assert_called_once_with(dest="subcommand")
    subparsers.add_parser.assert_has_calls([call("jptext")])
    assert actual == parser


def test_main_print_help(mocker: MockerFixture) -> None:
    setup_argument_parser = mocker.patch("jittok.cli.setup_argument_parser")
    parser = setup_argument_parser.return_value
    args = parser.parse_args.return_value
    args.subcommand = None
    cli.main()
    setup_argument_parser.assert_called_once_with()
    parser.parse_args.assert_called_once_with()
    parser.print_help.assert_called_once_with()


def test_main_jptext(mocker: MockerFixture) -> None:
    setup_argument_parser = mocker.patch("jittok.cli.setup_argument_parser")
    jptext_cli = mocker.patch("jittok.cli.jptext_cli")
    parser = setup_argument_parser.return_value
    args = parser.parse_args.return_value
    args.subcommand = "jptext"
    cli.main()
    setup_argument_parser.assert_called_once_with()
    parser.parse_args.assert_called_once_with()
    parser.print_help.assert_not_called()
    jptext_cli.main.assert_called_once_with(args)



from argparse import ArgumentParser

from .jptext import cli as jptext_cli


def main() -> None:
    parser = setup_argument_parser()
    args = parser.parse_args()
    if args.subcommand == "jptext":
        jptext_cli.main(args)
    else:
        parser.print_help()


def setup_argument_parser() -> ArgumentParser:
    parser = ArgumentParser()
    subparsers = parser.add_subparsers(dest="subcommand")
    jptext_cli.setup_argument_subparser(subparsers.add_parser("jptext"))
    return parser



__version__ = "0.0.2"
__author__ = "osoken"
__description__ = "Swiss Army Knife-like toolbox for data processing"
__email__ = "osoken.devel@outlook.jp"
__package_name__ = "jittok"


from .core import Callable

__all__ = ["Callable"]



import os

from setuptools import setup

from jittok import __author__, __description__, __email__, __package_name__, __version__

with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"), "r") as fin:
    __long_description__ = fin.read()

setup(
    name=__package_name__,
    version=__version__,
    author=__author__,
    author_email=__email__,
    license="MIT",
    url="https://github.com/osoken/jittok",
    description=__description__,
    long_description=__long_description__,
    long_description_content_type="text/markdown",
    package_data={__package_name__: ["py.typed"]},
    packages=[__package_name__, f"{__package_name__}.jptext"],
    install_requires=["regex"],
    extras_require={
        "dev": ["flake8", "pytest", "black", "mypy", "tox", "isort", "types-regex", "pytest-mock"],
    },
)



from abc import ABCMeta, abstractmethod
from typing import Generic, TypeVar

S = TypeVar("S")
T = TypeVar("T")


class Callable(Generic[S, T], metaclass=ABCMeta):
    @abstractmethod
    def __call__(self, args: S) -> T:
        ...
